@@ -1,6 +1,6 @@
 //! JWS Cryptographic Operations
 
-use openssl::{bn, ec, ecdsa, hash, nid, pkey};
+use openssl::{bn, ec, ecdsa, hash, nid, pkey, rsa, sign};
 use serde::{Deserialize, Serialize};
 use std::convert::TryFrom;
 use std::fmt;
@@ -10,6 +10,9 @@ use url::Url;
 use crate::base64_data::Base64UrlSafeData;
 use crate::error::JwtError;
 
+const RSA_MIN_SIZE: u32 = 3072;
+const RSA_SIG_SIZE: i32 = 384;
+
 // https://datatracker.ietf.org/doc/html/rfc7515
 
 #[derive(Debug, Serialize, Clone, Deserialize)]
@@ -54,6 +57,22 @@ pub enum Jwk {
         /// The key id
         kid: Option<String>,
     },
+    /// Legacy RSA public key
+    RSA {
+        /// Public n value
+        n: Base64UrlSafeData,
+        /// Public exponent
+        e: Base64UrlSafeData,
+        /// The algorithm in use for this key
+        #[serde(skip_serializing_if = "Option::is_none")]
+        alg: Option<JwaAlg>,
+        #[serde(rename = "use", skip_serializing_if = "Option::is_none")]
+        /// The usage of this key
+        use_: Option<JwkUse>,
+        #[serde(skip_serializing_if = "Option::is_none")]
+        /// The key id
+        kid: Option<String>,
+    },
 }
 
 #[derive(Debug, Serialize, Clone, Deserialize, PartialEq)]
@@ -72,6 +91,8 @@ pub enum JwkUse {
 pub enum JwaAlg {
     /// ECDSA with P-256 and SHA256
     ES256,
+    /// RSASSA-PKCS1-v1_5 with SHA-256
+    RS256,
 }
 
 #[derive(Clone)]
@@ -84,6 +105,13 @@ pub enum JwsSigner {
         /// The matching digest.
         digest: hash::MessageDigest,
     },
+    /// RSASSA-PKCS1-v1_5 with SHA-256
+    RS256 {
+        /// Private Key
+        skey: rsa::Rsa<pkey::Private>,
+        /// The matching digest.
+        digest: hash::MessageDigest,
+    },
 }
 
 #[derive(Clone)]
@@ -96,6 +124,13 @@ pub enum JwsValidator {
         /// The matching digest.
         digest: hash::MessageDigest,
     },
+    /// RSASSA-PKCS1-v1_5 with SHA-256
+    RS256 {
+        /// Public Key
+        pkey: rsa::Rsa<pkey::Public>,
+        /// The matching digest.
+        digest: hash::MessageDigest,
+    },
 }
 
 impl fmt::Debug for JwsValidator {
@@ -215,6 +250,7 @@ impl Jws {
     ) -> Result<JwsCompact, JwtError> {
         let alg = match signer {
             JwsSigner::ES256 { skey: _, digest: _ } => JwaAlg::ES256,
+            JwsSigner::RS256 { skey: _, digest: _ } => JwaAlg::RS256,
         };
 
         let header = ProtectedHeader {
@@ -238,37 +274,52 @@ impl Jws {
             .map(|bytes| base64::encode_config(&bytes, base64::URL_SAFE_NO_PAD))?;
         let payload_b64 = base64::encode_config(&self.payload, base64::URL_SAFE_NO_PAD);
 
-        // eprintln!("sinput -> {}", format!("{}.{}", hdr_b64, payload_b64));
+        // trace!("sinput -> {}", format!("{}.{}", hdr_b64, payload_b64));
 
         let sign_input = format!("{}.{}", hdr_b64, payload_b64).as_bytes().to_vec();
 
-        // eprintln!("sinput -> {:?}", sign_input);
+        trace!("sinput -> {:?}", sign_input);
 
         // Compute the signature!
-        let ec_sig = match signer {
+        let signature = match signer {
             JwsSigner::ES256 { skey, digest } => {
                 let hashout =
                     hash::hash(*digest, &sign_input).map_err(|_| JwtError::OpenSSLError)?;
-                ecdsa::EcdsaSig::sign(&hashout, &skey)
+                let ec_sig =
+                    ecdsa::EcdsaSig::sign(&hashout, &skey).map_err(|_| JwtError::OpenSSLError)?;
+
+                let mut r = [0; 32];
+                let r_vec = ec_sig.r().to_vec();
+                let (_left, right) = r.split_at_mut(32 - r_vec.len());
+                right.copy_from_slice(r_vec.as_slice());
+                let mut s = [0; 32];
+                let s_vec = ec_sig.s().to_vec();
+                let (_left, right) = s.split_at_mut(32 - s_vec.len());
+                right.copy_from_slice(s_vec.as_slice());
+
+                // trace!("r {:?}", r);
+                // trace!("s {:?}", s);
+
+                let mut signature = Vec::with_capacity(64);
+                signature.extend_from_slice(&r);
+                signature.extend_from_slice(&s);
+                signature
             }
-        }
-        .map_err(|_| JwtError::OpenSSLError)?;
+            JwsSigner::RS256 { skey, digest } => {
+                let key = pkey::PKey::from_rsa(skey.clone()).map_err(|_| JwtError::OpenSSLError)?;
 
-        let mut r = [0; 32];
-        let r_vec = ec_sig.r().to_vec();
-        let (_left, right) = r.split_at_mut(32 - r_vec.len());
-        right.copy_from_slice(r_vec.as_slice());
-        let mut s = [0; 32];
-        let s_vec = ec_sig.s().to_vec();
-        let (_left, right) = s.split_at_mut(32 - s_vec.len());
-        right.copy_from_slice(s_vec.as_slice());
+                let mut signer =
+                    sign::Signer::new(*digest, &key).map_err(|_| JwtError::OpenSSLError)?;
 
-        // eprintln!("r {:?}", r);
-        // eprintln!("s {:?}", s);
+                signer
+                    .set_rsa_padding(rsa::Padding::PKCS1)
+                    .map_err(|_| JwtError::OpenSSLError)?;
 
-        let mut signature = Vec::with_capacity(64);
-        signature.extend_from_slice(&r);
-        signature.extend_from_slice(&s);
+                signer
+                    .sign_oneshot_to_vec(&sign_input)
+                    .map_err(|_| JwtError::OpenSSLError)?
+            }
+        };
 
         Ok(JwsCompact {
             header,
@@ -333,7 +384,38 @@ impl JwsCompact {
                 } else {
                     Err(JwtError::InvalidSignature)
                 }
-            } // _ => Err(JwtError::ValidatorAlgMismatch),
+            }
+            (JwsValidator::RS256 { pkey, digest }, JwaAlg::RS256) => {
+                if self.signature.len() < 256 {
+                    return Err(JwtError::InvalidSignature);
+                }
+
+                let p = pkey::PKey::from_rsa(pkey.clone()).map_err(|_| JwtError::OpenSSLError)?;
+
+                let mut verifier =
+                    sign::Verifier::new(*digest, &p).map_err(|_| JwtError::OpenSSLError)?;
+                verifier
+                    .set_rsa_padding(rsa::Padding::PKCS1)
+                    .map_err(|_| JwtError::OpenSSLError)?;
+
+                verifier
+                    .update(&self.sign_input)
+                    .map_err(|_| JwtError::OpenSSLError)?;
+                verifier
+                    .verify(&self.signature)
+                    .map_err(|_| JwtError::OpenSSLError)
+                    .and_then(|res| {
+                        if res {
+                            Ok(Jws {
+                                header: (&self.header).into(),
+                                payload: self.payload.clone(),
+                            })
+                        } else {
+                            Err(JwtError::InvalidSignature)
+                        }
+                    })
+            }
+            _ => Err(JwtError::ValidatorAlgMismatch),
         }
     }
 }
@@ -432,6 +514,23 @@ impl TryFrom<&Jwk> for JwsValidator {
                     EcCurve::P256 => JwsValidator::ES256 { pkey, digest },
                 })
             }
+            Jwk::RSA {
+                n,
+                e,
+                alg: _,
+                use_: _,
+                kid: _,
+            } => {
+                let digest = hash::MessageDigest::sha256();
+
+                let nbn = bn::BigNum::from_slice(&n.0).map_err(|_| JwtError::OpenSSLError)?;
+                let ebn = bn::BigNum::from_slice(&e.0).map_err(|_| JwtError::OpenSSLError)?;
+
+                let pkey = rsa::Rsa::from_public_components(nbn, ebn)
+                    .map_err(|_| JwtError::OpenSSLError)?;
+
+                Ok(JwsValidator::RS256 { pkey, digest })
+            }
         }
     }
 }
@@ -478,6 +577,16 @@ impl JwsSigner {
                         digest: digest.clone(),
                     })
             }
+            JwsSigner::RS256 { skey, digest } => {
+                let n = skey.n().to_owned().map_err(|_| JwtError::OpenSSLError)?;
+                let e = skey.e().to_owned().map_err(|_| JwtError::OpenSSLError)?;
+                rsa::Rsa::from_public_components(n, e)
+                    .map_err(|_| JwtError::OpenSSLError)
+                    .map(|pkey| JwsValidator::RS256 {
+                        pkey,
+                        digest: digest.clone(),
+                    })
+            }
         }
     }
 
@@ -491,6 +600,16 @@ impl JwsSigner {
         })
     }
 
+    /// Restore this JwsSigner from a DER private key.
+    pub fn from_rs256_der(der: &[u8]) -> Result<Self, JwtError> {
+        let skey = rsa::Rsa::private_key_from_der(der).map_err(|_| JwtError::OpenSSLError)?;
+
+        Ok(JwsSigner::RS256 {
+            skey,
+            digest: hash::MessageDigest::sha256(),
+        })
+    }
+
     /*
     pub fn public_key_to_der(&self) -> Result<Vec<u8>, JwtError> {
         unimplemented!();
@@ -503,6 +622,9 @@ impl JwsSigner {
             JwsSigner::ES256 { skey, digest: _ } => skey
                 .private_key_to_der()
                 .map_err(|_| JwtError::OpenSSLError),
+            JwsSigner::RS256 { skey, digest: _ } => skey
+                .private_key_to_der()
+                .map_err(|_| JwtError::OpenSSLError),
         }
     }
 
@@ -520,6 +642,17 @@ impl JwsSigner {
         })
     }
 
+    /// Create a new legacy (RSA) private key for signing
+    pub fn generate_legacy_rs256() -> Result<Self, JwtError> {
+        let skey = rsa::Rsa::generate(RSA_MIN_SIZE).map_err(|_| JwtError::OpenSSLError)?;
+
+        skey.check_key().map_err(|_| JwtError::OpenSSLError)?;
+        Ok(JwsSigner::RS256 {
+            skey,
+            digest: hash::MessageDigest::sha256(),
+        })
+    }
+
     /// Export the public key of this signer as a Jwk
     pub fn public_key_as_jwk(&self, kid: Option<&str>) -> Result<Jwk, JwtError> {
         match self {
@@ -560,6 +693,25 @@ impl JwsSigner {
                     kid: kid.map(str::to_string),
                 })
             }
+            JwsSigner::RS256 { skey, digest: _ } => {
+                let public_key_n = skey
+                    .n()
+                    .to_vec_padded(RSA_SIG_SIZE)
+                    .map_err(|_| JwtError::OpenSSLError)?;
+
+                let public_key_e = skey
+                    .e()
+                    .to_vec_padded(3)
+                    .map_err(|_| JwtError::OpenSSLError)?;
+
+                Ok(Jwk::RSA {
+                    n: Base64UrlSafeData(public_key_n),
+                    e: Base64UrlSafeData(public_key_e),
+                    alg: Some(JwaAlg::RS256),
+                    use_: Some(JwkUse::Sig),
+                    kid: kid.map(str::to_string),
+                })
+            }
         }
     }
 }
@@ -571,7 +723,8 @@ mod tests {
     use std::str::FromStr;
 
     #[test]
-    fn rfc7515_validation_example() {
+    fn rfc7515_es256_validation_example() {
+        let _ = tracing_subscriber::fmt().try_init();
         let test_jws = "eyJhbGciOiJFUzI1NiJ9.eyJpc3MiOiJqb2UiLA0KICJleHAiOjEzMDA4MTkzODAsDQogImh0dHA6Ly9leGFtcGxlLmNvbS9pc19yb290Ijp0cnVlfQ.DtEhU3ljbEg8L38VWAfUAqOyKAM6-Xx-F4GawxaepmXFCgfTjDxw5djxLa8ISlSApmWQxfKTUJqPP3-Kg6NU1Q";
 
         let jwsc = JwsCompact::from_str(test_jws).unwrap();
@@ -602,7 +755,7 @@ mod tests {
         let pkey = r#"{"kty":"EC","crv":"P-256","x":"f83OJ3D2xF1Bg8vub9tLe1gHMzV76e8Tus9uPHvRVEU","y":"x_FEzRu9m36HLN_tue659LNpXW6pCyStikYjKIWI5a0"}"#;
 
         let pkey: Jwk = serde_json::from_str(pkey).expect("Invalid JWK");
-        eprintln!("jwk -> {:?}", pkey);
+        trace!("jwk -> {:?}", pkey);
 
         let jws_validator = JwsValidator::try_from(&pkey).expect("Unable to create validator");
         assert!(jwsc.get_jwk_pubkey_url().is_none());
@@ -610,11 +763,12 @@ mod tests {
         let released = jwsc
             .validate(&jws_validator)
             .expect("Unable to validate jws");
-        eprintln!("rel -> {:?}", released);
+        trace!("rel -> {:?}", released);
     }
 
     #[test]
-    fn rfc7515_signature_example() {
+    fn rfc7515_es256_signature_example() {
+        let _ = tracing_subscriber::fmt().try_init();
         // https://docs.rs/openssl/0.10.36/openssl/ec/struct.EcKey.html#method.from_private_components
         let jwss = JwsSigner::from_es256_jwk_components(
             "f83OJ3D2xF1Bg8vub9tLe1gHMzV76e8Tus9uPHvRVEU",
@@ -638,18 +792,18 @@ mod tests {
         let pkey = r#"{"kty":"EC","crv":"P-256","x":"f83OJ3D2xF1Bg8vub9tLe1gHMzV76e8Tus9uPHvRVEU","y":"x_FEzRu9m36HLN_tue659LNpXW6pCyStikYjKIWI5a0"}"#;
 
         let pkey: Jwk = serde_json::from_str(pkey).expect("Invalid JWK");
-        eprintln!("jwk -> {:?}", pkey);
+        trace!("jwk -> {:?}", pkey);
 
         let jws_validator = JwsValidator::try_from(&pkey).expect("Unable to create validator");
 
         let released = jwsc
             .validate(&jws_validator)
             .expect("Unable to validate jws");
-        eprintln!("rel -> {:?}", released);
+        trace!("rel -> {:?}", released);
     }
 
     #[test]
-    fn key_generate_cycle() {
+    fn es256_key_generate_cycle() {
         let jwss = JwsSigner::generate_es256().expect("failed to construct signer.");
 
         let der = jwss.private_key_to_der().expect("Failed to extract DER");
@@ -676,6 +830,96 @@ mod tests {
         assert!(released.payload() == &[0, 1, 2, 3, 4]);
     }
 
+    // RSA3072
+    // https://datatracker.ietf.org/doc/html/rfc7515#appendix-A.2
+    #[test]
+    fn rfc7515_rs256_validation_example() {
+        let _ = tracing_subscriber::fmt().try_init();
+        let test_jws = "eyJhbGciOiJSUzI1NiJ9.eyJpc3MiOiJqb2UiLA0KICJleHAiOjEzMDA4MTkzODAsDQogImh0dHA6Ly9leGFtcGxlLmNvbS9pc19yb290Ijp0cnVlfQ.cC4hiUPoj9Eetdgtv3hF80EGrhuB__dzERat0XF9g2VtQgr9PJbu3XOiZj5RZmh7AAuHIm4Bh-0Qc_lF5YKt_O8W2Fp5jujGbds9uJdbF9CUAr7t1dnZcAcQjbKBYNX4BAynRFdiuB--f_nZLgrnbyTyWzO75vRK5h6xBArLIARNPvkSjtQBMHlb1L07Qe7K0GarZRmB_eSN9383LcOLn6_dO--xi12jzDwusC-eOkHWEsqtFZESc6BfI7noOPqvhJ1phCnvWh6IeYI2w9QOYEUipUTI8np6LbgGY9Fs98rqVt5AXLIhWkWywlVmtVrBp0igcN_IoypGlUPQGe77Rw";
+
+        let jwsc = JwsCompact::from_str(test_jws).unwrap();
+
+        assert!(jwsc.to_string() == test_jws);
+
+        assert!(jwsc.check_vectors(
+            &[
+                101, 121, 74, 104, 98, 71, 99, 105, 79, 105, 74, 83, 85, 122, 73, 49, 78, 105, 74,
+                57, 46, 101, 121, 74, 112, 99, 51, 77, 105, 79, 105, 74, 113, 98, 50, 85, 105, 76,
+                65, 48, 75, 73, 67, 74, 108, 101, 72, 65, 105, 79, 106, 69, 122, 77, 68, 65, 52,
+                77, 84, 107, 122, 79, 68, 65, 115, 68, 81, 111, 103, 73, 109, 104, 48, 100, 72, 65,
+                54, 76, 121, 57, 108, 101, 71, 70, 116, 99, 71, 120, 108, 76, 109, 78, 118, 98, 83,
+                57, 112, 99, 49, 57, 121, 98, 50, 57, 48, 73, 106, 112, 48, 99, 110, 86, 108, 102,
+                81
+            ],
+            &[
+                112, 46, 33, 137, 67, 232, 143, 209, 30, 181, 216, 45, 191, 120, 69, 243, 65, 6,
+                174, 27, 129, 255, 247, 115, 17, 22, 173, 209, 113, 125, 131, 101, 109, 66, 10,
+                253, 60, 150, 238, 221, 115, 162, 102, 62, 81, 102, 104, 123, 0, 11, 135, 34, 110,
+                1, 135, 237, 16, 115, 249, 69, 229, 130, 173, 252, 239, 22, 216, 90, 121, 142, 232,
+                198, 109, 219, 61, 184, 151, 91, 23, 208, 148, 2, 190, 237, 213, 217, 217, 112, 7,
+                16, 141, 178, 129, 96, 213, 248, 4, 12, 167, 68, 87, 98, 184, 31, 190, 127, 249,
+                217, 46, 10, 231, 111, 36, 242, 91, 51, 187, 230, 244, 74, 230, 30, 177, 4, 10,
+                203, 32, 4, 77, 62, 249, 18, 142, 212, 1, 48, 121, 91, 212, 189, 59, 65, 238, 202,
+                208, 102, 171, 101, 25, 129, 253, 228, 141, 247, 127, 55, 45, 195, 139, 159, 175,
+                221, 59, 239, 177, 139, 93, 163, 204, 60, 46, 176, 47, 158, 58, 65, 214, 18, 202,
+                173, 21, 145, 18, 115, 160, 95, 35, 185, 232, 56, 250, 175, 132, 157, 105, 132, 41,
+                239, 90, 30, 136, 121, 130, 54, 195, 212, 14, 96, 69, 34, 165, 68, 200, 242, 122,
+                122, 45, 184, 6, 99, 209, 108, 247, 202, 234, 86, 222, 64, 92, 178, 33, 90, 69,
+                178, 194, 85, 102, 181, 90, 193, 167, 72, 160, 112, 223, 200, 163, 42, 70, 149, 67,
+                208, 25, 238, 251, 71
+            ]
+        ));
+
+        assert!(jwsc.get_jwk_pubkey_url().is_none());
+        assert!(jwsc.get_jwk_pubkey().is_none());
+
+        let pkey = r#"{
+            "kty":"RSA",
+            "n":"ofgWCuLjybRlzo0tZWJjNiuSfb4p4fAkd_wWJcyQoTbji9k0l8W26mPddxHmfHQp-Vaw-4qPCJrcS2mJPMEzP1Pt0Bm4d4QlL-yRT-SFd2lZS-pCgNMsD1W_YpRPEwOWvG6b32690r2jZ47soMZo9wGzjb_7OMg0LOL-bSf63kpaSHSXndS5z5rexMdbBYUsLA9e-KXBdQOS-UTo7WTBEMa2R2CapHg665xsmtdVMTBQY4uDZlxvb3qCo5ZwKh9kG4LT6_I5IhlJH7aGhyxXFvUK-DWNmoudF8NAco9_h9iaGNj8q2ethFkMLs91kzk2PAcDTW9gb54h4FRWyuXpoQ",
+            "e":"AQAB"
+        }"#;
+
+        let pkey: Jwk = serde_json::from_str(pkey).expect("Invalid JWK");
+        trace!("jwk -> {:?}", pkey);
+
+        let jws_validator = JwsValidator::try_from(&pkey).expect("Unable to create validator");
+        assert!(jwsc.get_jwk_pubkey_url().is_none());
+
+        let released = jwsc
+            .validate(&jws_validator)
+            .expect("Unable to validate jws");
+        trace!("rel -> {:?}", released);
+    }
+
+    #[test]
+    fn rs256_key_generate_cycle() {
+        let _ = tracing_subscriber::fmt().try_init();
+        let jwss = JwsSigner::generate_legacy_rs256().expect("failed to construct signer.");
+
+        let der = jwss.private_key_to_der().expect("Failed to extract DER");
+
+        let jwss = JwsSigner::from_rs256_der(&der).expect("Failed to restore signer");
+
+        // This time we'll add the jwk pubkey and show it being used with the validator.
+        let jws = Jws::new(vec![0, 1, 2, 3, 4])
+            .set_kid("abcd".to_string())
+            .set_typ("abcd".to_string())
+            .set_cty("abcd".to_string());
+
+        let jwsc = jws.sign_embed_public_jwk(&jwss).expect("Failed to sign");
+
+        assert!(jwsc.get_jwk_pubkey_url().is_none());
+        let pub_jwk = jwsc.get_jwk_pubkey().expect("No embeded public jwk!");
+        assert!(*pub_jwk == jwss.public_key_as_jwk(None).unwrap());
+
+        let jws_validator = JwsValidator::try_from(pub_jwk).expect("Unable to create validator");
+
+        let released = jwsc
+            .validate(&jws_validator)
+            .expect("Unable to validate jws");
+        assert!(released.payload() == &[0, 1, 2, 3, 4]);
+    }
+
     // A test for the signer to/from der.
     // directly get the validator from the signer.
 }
